@@ -776,7 +776,7 @@ void SuperAlignment::getSitePatternIndex(IntVector &pattern_index) {
 
 void SuperAlignment::getPatternFreq(IntVector &pattern_freq) {
 	ASSERT(isSuperAlignment());
-	int offset = 0;
+	size_t offset = 0;
 	if (!pattern_freq.empty()) pattern_freq.resize(0);
 	for (vector<Alignment*>::iterator it = partitions.begin(); it != partitions.end(); it++) {
 		IntVector freq;
@@ -786,6 +786,15 @@ void SuperAlignment::getPatternFreq(IntVector &pattern_freq) {
 	}
 }
 
+void SuperAlignment::getPatternFreq(int *pattern_freq) {
+    ASSERT(isSuperAlignment());
+    size_t offset = 0;
+    for (vector<Alignment*>::iterator it = partitions.begin(); it != partitions.end(); it++) {
+        (*it)->getPatternFreq(pattern_freq + offset);
+        offset += (*it)->getNPattern();
+    }
+}
+
 void SuperAlignment::printSiteInfo(const char* filename) {
     try {
         ofstream out(filename);
@@ -799,6 +808,7 @@ void SuperAlignment::printSiteInfo(const char* filename) {
     }
 }
 
+/*
 void SuperAlignment::createBootstrapAlignment(Alignment *aln, IntVector* pattern_freq, const char *spec) {
 	ASSERT(aln->isSuperAlignment());
 	Alignment::copyAlignment(aln);
@@ -881,6 +891,71 @@ void SuperAlignment::createBootstrapAlignment(Alignment *aln, IntVector* pattern
 	taxa_index = super_aln->taxa_index;
     countConstSite();
 }
+*/
+
+void SuperAlignment::createBootstrapAlignment(Alignment *aln, IntVector* pattern_freq, const char *spec) {
+    ASSERT(aln->isSuperAlignment());
+    SuperAlignment *super_aln = (SuperAlignment*) aln;
+    ASSERT(partitions.empty());
+    name = aln->name;
+    model_name = aln->model_name;
+    sequence_type = aln->sequence_type;
+    position_spec = aln->position_spec;
+    aln_file = aln->aln_file;
+    
+    if (!spec) {
+        // resampling sites within genes
+        Alignment::copyAlignment(aln);
+        partitions.reserve(super_aln->partitions.size());
+        for (vector<Alignment*>::iterator it = super_aln->partitions.begin(); it != super_aln->partitions.end(); it++) {
+            Alignment *boot_aln = new Alignment;
+            if (pattern_freq) {
+                IntVector part_pattern_freq;
+                boot_aln->createBootstrapAlignment(*it, &part_pattern_freq);
+                pattern_freq->insert(pattern_freq->end(), part_pattern_freq.begin(), part_pattern_freq.end());
+            } else {
+                boot_aln->createBootstrapAlignment(*it);
+            }
+            partitions.push_back(boot_aln);
+        }
+        taxa_index = super_aln->taxa_index;
+        countConstSite();
+    } else if (strcmp(spec, "GENE") == 0) {
+        ASSERT(!pattern_freq);
+        // resampling whole genes
+        IntVector gene_freq;
+        random_resampling(super_aln->partitions.size(), gene_freq);
+        for (int i = 0; i < gene_freq.size(); i++)
+            if (gene_freq[i] > 0) {
+                Alignment *boot_aln = new Alignment;
+                boot_aln->copyAlignment(super_aln->partitions[i]);
+                if (gene_freq[i] > 1) {
+                    for (auto it = boot_aln->begin(); it != boot_aln->end(); it++)
+                        it->frequency *= gene_freq[i];
+                    auto site_pattern = boot_aln->site_pattern;
+                    for (int j = 1; j < gene_freq[i]; j++)
+                        boot_aln->site_pattern.insert(boot_aln->site_pattern.end(), site_pattern.begin(), site_pattern.end());
+                }
+                partitions.push_back(boot_aln);
+            }
+        init();
+    } else if (strcmp(spec, "GENESITE") == 0) {
+        ASSERT(!pattern_freq);
+        // resampling whole genes then sites within resampled genes
+        IntVector gene_freq;
+        random_resampling(super_aln->partitions.size(), gene_freq);
+        for (int i = 0; i < gene_freq.size(); i++)
+            for (int rep = 0; rep < gene_freq[i]; rep++) {
+            Alignment *boot_aln = new Alignment;
+            boot_aln->createBootstrapAlignment(super_aln->partitions[i]);
+            boot_aln->name = boot_aln->name + "." + convertIntToString(rep);
+            partitions.push_back(boot_aln);
+        }
+        init();
+    } else {
+        outError("Wrong -bsam, either -bsam GENE or -bsam GENESITE");
+    }
+}
 
 void SuperAlignment::createBootstrapAlignment(IntVector &pattern_freq, const char *spec) {
 	ASSERT(isSuperAlignment());
@@ -910,19 +985,23 @@ void SuperAlignment::createBootstrapAlignment(int *pattern_freq, const char *spe
 			nptn += (*it)->getNPattern();
 		}
 		memset(pattern_freq, 0, nptn * sizeof(int));
-		for (int i = 0; i < partitions.size(); i++) {
-			int part = random_int(partitions.size(), rstream);
+        IntVector gene_freq;
+        random_resampling(partitions.size(), gene_freq, rstream);
+		for (int part = 0; part < partitions.size(); part++)
+        for (int rep = 0; rep < gene_freq[part]; rep++){
 			Alignment *aln = partitions[part];
 			if (strncmp(spec,"GENESITE",8) == 0) {
 				// then resampling sites in resampled gene
-				for (int j = 0; j < aln->getNSite(); j++) {
-					int ptn_id = aln->getPatternID(random_int(aln->getNPattern(), rstream));
+                IntVector sample;
+                random_resampling(aln->getNSite(), sample, rstream);
+				for (int site = 0; site < sample.size(); site++)
+                for (int rep2 = 0; rep2 < sample[site]; rep2++) {
+					int ptn_id = aln->getPatternID(site);
 					pattern_freq[ptn_id + part_pos[part]]++;
 				}
-
 			} else {
-				for (int j = 0; j < aln->getNPattern(); j++)
-					pattern_freq[j + part_pos[part]] += aln->at(j).frequency;
+				for (int ptn = 0; ptn < aln->getNPattern(); ptn++)
+					pattern_freq[ptn + part_pos[part]] += aln->at(ptn).frequency;
 			}
 		}
 	} else {
@@ -1148,21 +1227,25 @@ Alignment *SuperAlignment::concatenateAlignments(set<int> &ids) {
     int site = 0;
     for (it = ids.begin(); it != ids.end(); it++) {
     	int id = *it;
-		string taxa_set;
-        Pattern taxa_pat = getPattern(id);
-        taxa_set.insert(taxa_set.begin(), taxa_pat.begin(), taxa_pat.end());
+        // 2018-08-23: important bugfix in v1.6: taxa_set has wrong correspondance
+		//string taxa_set;
+        //Pattern taxa_pat = getPattern(id);
+        //taxa_set.insert(taxa_set.begin(), taxa_pat.begin(), taxa_pat.end());
     	for (Alignment::iterator it = partitions[id]->begin(); it != partitions[id]->end(); it++) {
     		Pattern pat;
-    		int part_seq = 0;
+    		//int part_seq = 0;
     		for (int seq = 0; seq < union_taxa.size(); seq++)
     			if (union_taxa[seq] == 1) {
     				char ch = aln->STATE_UNKNOWN;
-    				if (taxa_set[seq] == 1) {
-    					ch = (*it)[part_seq++];
-    				}
+                    int seq_part = taxa_index[seq][id];
+                    if (seq_part >= 0)
+                        ch = (*it)[seq_part];
+                    //if (taxa_set[seq] == 1) {
+                    //    ch = (*it)[part_seq++];
+                    //}
     				pat.push_back(ch);
     			}
-    		ASSERT(part_seq == partitions[id]->getNSeq());
+    		//ASSERT(part_seq == partitions[id]->getNSeq());
     		aln->addPattern(pat, site, (*it).frequency);
     		// IMPORTANT BUG FIX FOLLOW
     		int ptnindex = aln->pattern_index[pat];
@@ -1179,11 +1262,12 @@ Alignment *SuperAlignment::concatenateAlignments(set<int> &ids) {
 
 Alignment *SuperAlignment::concatenateAlignments() {
     vector<SeqType> seq_types;
+    vector<char*> genetic_codes;
     vector<set<int> > ids;
     for (int i = 0; i < partitions.size(); i++) {
         bool found = false;
         for (int j = 0; j < seq_types.size(); j++)
-            if (partitions[i]->seq_type == seq_types[j]) {
+            if (partitions[i]->seq_type == seq_types[j] && partitions[i]->genetic_code == genetic_codes[j]) {
                 ids[j].insert(i);
                 found = true;
                 break;
@@ -1192,6 +1276,7 @@ Alignment *SuperAlignment::concatenateAlignments() {
             continue;
         // create a new partition
         seq_types.push_back(partitions[i]->seq_type);
+        genetic_codes.push_back(partitions[i]->genetic_code);
         ids.push_back(set<int>());
         ids.back().insert(i);
     }
